/*
DL2 Monitor: ./pa/paudp.c
Copyright (C) 2002, Rolf Jger, Ulm (Germany)

See the file README for information on usage and redistribution
of this source code file, and for a DISCLAIMER OF ALL WARRANTIES.
*/

/*----------------------------------------------------------------------------
 * Datalink Monitor (paudp): Protocol Analysis (RFC768)
 *
 * Implementation Notes
 * default contains: 
 * default contains: source-port, destination-port.
 * 
 * Author: Rolf Jger,Ulm
 *----------------------------------------------------------------------------
 * YYYYMMDD  History
 * 20020427  implemented according to rfc768.
 * 20020330  created from scratch
 * $Id: paudp.c,v 1.5 2002/04/29 18:05:48 dv Exp $
 *----------------------------------------------------------------------------
*/
# include <stdio.h>
# include <string.h>
# include "lib/dl2pa.h"
# include "pa/portmap.h"
# include "pa/udp.h"

// UDP frame positions:
# define P_SOURCE_PORT       0x00
# define P_DESTINATION_PORT  0x02
# define P_LENGTH            0x04
# define P_CHECKSUM          0x06
# define P_DATA              0x08

//-exports--------------------------------------------------------------------
int dl2_paudp(Dl2_Frame *frame,int verbose) {
  Octet *buffer = frame->buffer;
  int length = frame->length, i = P_SOURCE_PORT;

  // check for required packet length:
  if (length < P_DATA) return -1;

  //source-port:
  if (verbose & UDP_Msource_port) {
    printf(" %s=%02x%02x",UDP_Fsource_port,buffer[i],buffer[i+1]);
  }
  else if (verbose & UDP_Mdefault) {
    Doctet pn = str2doct(buffer+i);
    const char *pnstr = PA_PORTNAME(pn);
    if (pnstr != NULL) printf(" %s",pnstr);
    else printf(" %02x%02x",buffer[i],buffer[i+1]);
  } i = P_DESTINATION_PORT;

  //destination-port:
  if (verbose & UDP_Mdestination_port) {
    printf(" %s=%02x%02x",UDP_Fdestination_port,buffer[i],buffer[i+1]);
  }
  else if (verbose & UDP_Mdefault) {
    Doctet pn = str2doct(buffer+i);
    const char *pnstr = PA_PORTNAME(pn);
    if (pnstr != NULL) printf(" %s",pnstr);
    else printf(" %02x%02x",buffer[i],buffer[i+1]);
  } i = P_LENGTH;

  // length:
  if (verbose & UDP_Mlength) {
    printf(" %s=%02x%02x",UDP_Flength,buffer[i],buffer[i+1]);
  } i = P_CHECKSUM;

  // checksum:
  if (verbose & UDP_Mchecksum) {
    printf(" %s=%02x%02x",UDP_Fchecksum,buffer[i],buffer[i+1]);
  } i = P_DATA;

  // data:
  if (verbose & UDP_Mdata) {
    Doctet dlength = str2doct(buffer+P_LENGTH);
    if (dlength > length) dlength = length;
    printf(" %s=",UDP_Fdata); 
    for (;i < dlength; i++) printf("%02x",buffer[i]);   
  } i = length;

  return i;
} //dl2_paudp


//-locals---------------------------------------------------------------------

/***EOM***/
